%===============================================================================
%      Romualdo Alejandro Ferreyra Ph.D.                     04.12.2022
%===============================================================================
% The following code calculates the ABCD parameters of a circuit composed of two
% branches of the "ladder-LC" type (ladder-LC network) in parallel, in which the
% elements L are connected in series and the elements C are in derivation in each
% one of the connecting nodes.
%====================
% GENERAL DESCRIPTION
%====================
% Everything referred to with the ladder branches will be denominated with M for
% one of them and with N for the other. The branch M will be composed of a number
% M1 of elements L1, M2 of elements L2 and a number M3 of elements L3, it will
% also have a number M1 of C1, M2 of C2 and an amount M3 of C3. The groups
% generated by the elements L1-C1, L2-C2 and L3-C3 will be called Na, Nb and Nc
% specifically What has been said for branch M applies in the same way for branch
% N but in this case the amounts of the different components are given by N1, N2,
% and N3.
%======================
% BEGINING OF THE CODE
%======================

    clc;        % clears all input and output from the Command Window
                % display, giving you a "clean screen".
    clear all;  % removes all variables from the workspace. This frees up
                % system memory.
%===================
% CONSTANTS
%===================
    M1 = 4;   % number of elements L1 and C1 in branch M.
    M2 = 4;   % number of elements L2 and C2 in branch M.
    M3 = 4;   % number of elements L3 and C3 in branch M.
    N1 = 4;   % number of elements L1 and C1 in branch N.
    N2 = 4;   % number of elements L2 and C2 in branch N.
    N3 = 4;   % number of L3 and C3 elements in branch N.

    FRECUENCIAINICIAL = 4;      % Frequencies from which
                                % begin to make the answer in frequency.
                                %
    FRECUENCIAFINAL = 11000;    % Frequencies at which
                                % egin to make the answer in frecuencia.

%===================
% VARIABLES
%===================
% Case I
      %L1 = 1e-3; % value of element L1 in Henry.
      %L2 = 1e-3; % value of element L2 in Henry.
      %L3 = 1e-3; % value of element L3 in Henry.
      %C1 = 1e-6; % value of element C1 in Faraday.
      %C2 = 1e-6; % value of element C2 in Faraday.
      %C3=  1e-6; % value of element C3 in Faraday.

%Case II
      %L1 = 1e-3; % value of element L1 in Henry.
      %L2 = 1e-3; % value of element L2 in Henry.
      %L3 = 1e-3;   % value of element L3 in Henry.
      %C1 = 1e-6;   % value of element C1 in Faraday.
      %C2 = 0.66e-6;   % value of element C2 in Faraday.
      %C3 = 1e-6;     % value of element C3 in Faraday.

%Case III
      L1 = 1.0e-3;  % value of element L1 in Henry.
      L2 = 0.0;     % value of element L2 in Henry.
      L3 = 1e-3;    % value of element L3 in Henry.
      C1 = 1e-6;    % value of element C1 in Faraday.
      C2 = 0.66e-6; % value of element C2 in Faraday.
      C3 = 1e-6;    % value of element C3 in Faraday.
%===================
% DEFINITIONS
%===================
 yM = [0];    % total admittance of the M branch as a function of frequency.
 yMaux = [0]; % auxiliary variable used for the formation of the row vector yM.
 yN = [0];    % total admittance of the M branch as a function of frequency.
 yNaux = [0]; % auxiliary variable used for the formation of the row vector yM.
 yT = [0];    % total admittance as a function of frequency, this is the
              % admittance resulting from the parallel of the two ladder branches.
 yTaux = [0]; % auxiliary variable used for the formation of the row vector yT.
%===============================================================================
% CYCLE "for"
% Through this loop you can find the values of yT for the
% different frequencies.
%===============================================================================
for a = FRECUENCIAINICIAL:2:FRECUENCIAFINAL + FRECUENCIAINICIAL-1 % Frequency
                                                               % responce range.
    s = 2*pi*a*i;                            % Complex frequency in radians.
%===========================
% ADMITTANCE AND IMPEDANCES
%===========================
%========================================================
% ADMITTANCE AND IMPEDANCES OF THE ELEMENTS OF BRANCH N
%========================================================
    Y2Na = (s*C1);
    Y2Nb = (s*C2);
    Y2Nc = (s*C3);
    Z1Na = (s*L1);
    Z1Nb = (s*L2);
    Z1Nc = (s*L3);
%================================================
% ADMITTANCE AND IMPEDANCES OF BRANCH ELEMENTS M
%================================================
    Y2Ma = (s*C1);
    Y2Mb = (s*C2);
    Y2Mc = (s*C3);
    Z1Ma = (s*L1);
    Z1Mb = (s*L2);
    Z1Mc = (s*L3);
%======================
% ABCD MATRIX ELEMENTS
%======================
%===========================================
% ELEMENTS OF THE ABCD MATRICES OF BRANCH N
%===========================================
%===========================================
% ELEMENTS OF THE ABCD MATRIXES OF GROUP Na
%===========================================
    N11a = (1+Z1Na*Y2Na);
    N12a = Z1Na;
    N21a = Y2Na;
    N22a = 1;
%===========================================
% ELEMENTS OF THE ABCD MATRIXES OF GROUP Nb
%===========================================
    N11b = (1+Z1Nb*Y2Nb);
    N12b = Z1Nb;
    N21b = Y2Nb;
    N22b = 1;
%=======================================
% ELEMENTS OF ABCD MATRIXES OF GROUP Nc
%=======================================
    N11c = (1+Z1Nc*Y2Nc);
    N12c = Z1Nc;
    N21c = Y2Nc;
    N22c = 1;
%============================================
% ELEMENTOS DE LA MATRICES ABCD DE LA RAMA M
%============================================
%============================================
% ELEMENTS OF THE ABCD MATRICES OF BRANCH Ma
%============================================
    M11a = 1;
    M12a = Z1Ma;
    M21a = Y2Ma;
    M22a = (1+Z1Ma*Y2Ma);
%============================================
% ELEMENTS OF THE ABCD MATRICES OF BRANCH Mb
%============================================
    M11b = 1;
    M12b = Z1Mb;
    M21b = Y2Mb;
    M22b = (1+Z1Mb*Y2Mb);
%============================================
% ELEMENTS OF THE ABCD MATRICES OF BRANCH Mc
%============================================
    M11c = 1;
    M12c = Z1Mc;
    M21c = Y2Mc;
    M22c = (1+Z1Mc*Y2Mc);
%===========================
% ABCD MATRICES OF BRANCH N
%===========================
    ABCDNa = [N11a N12a; N21a N22a];
    ABCDNb = [N11b N12b; N21b N22b];
    ABCDNc = [N11c N12c; N21c N22c];
%===========================
% ABCD MATRICES OF BRANCH M
%===========================
    ABCDMa = [M11a M12a; M21a M22a];
    ABCDMb = [M11b M12b; M21b M22b];
    ABCDMc = [M11c M12c; M21c M22c];
%=======================================================================
% ABCD MATRICES OF THE GROUPS OF BRANCH N RAISED TO POWERS N1,N2, AND N3
%=======================================================================
    ABCDNa = ABCDNa^ N1;
    ABCDNb = ABCDNb^ N2;
    ABCDNc = ABCDNc^ N3;
%========================================================================
% ABCD MATRICES OF THE M-BRANCH GROUPS RAISED TO THE POWERS M1,M2, AND M3
%========================================================================
    ABCDMa = ABCDMa^ M1;
    ABCDMb = ABCDMb^ M2;
    ABCDMc = ABCDMc^ M3;
%========================================================================
% MULTIPLICATION OF THE ABCD MATRICES OF THE GROUPS OF THE BRANCH N
% RAISED TO THE POWERS N1,N2, AND N3
%========================================================================
    ABCDNab = ABCDNa*ABCDNb;
    ABCDNabc = ABCDNab*ABCDNc;
%========================================================================
% MULTIPLICATION OF THE ABCD MATRICES OF THE GROUPS OF THE BRANCH M
% RAISED TO THE POWERS M1,M2 AND M3
%========================================================================
    ABCDMab = ABCDMa*ABCDMb;
    ABCDMabc = ABCDMab*ABCDMc;
%===================================================
% ELEMENTS OF THE ADMITTANCE MATRIX OF THE BRANCH N
%===================================================
    ABCDYN11 = ABCDNabc(2,2)/ABCDNabc(1,2);
    ABCDYN12 = -(ABCDNabc(1,1)*ABCDNabc(2,2)-ABCDNabc(1,2)*ABCDNabc(2,1))/ABCDNabc(1,2);
    ABCDYN21 = -1/ABCDNabc(1,2);
    ABCDYN22 = ABCDNabc(1,1)/ABCDNabc(1,2);
%===================================================
% ELEMENTS OF THE ADMITTANCE MATRIX OF THE BRANCH M
%===================================================
    ABCDYM11 = ABCDMabc(2,2)/ABCDMabc(1,2);
    ABCDYM12 = -(ABCDMabc(1,1)*ABCDMabc(2,2)-ABCDMabc(1,2)*ABCDMabc(2,1))/ABCDMabc(1,2);
    ABCDYM21 = -1/ABCDMabc(1,2);
    ABCDYM22 = ABCDMabc(1,1)/ABCDMabc(1,2);
%============================
% BRANCH ADMITTANCE MATRIX N
%============================
    ABCDYN = [ABCDYN11 ABCDYN12; ABCDYN21 ABCDYN22];
%============================
% BRANCH ADMITTANCE MATRIX M
%============================
    ABCDYM = [ABCDYM11 ABCDYM12; ABCDYM21 ABCDYM22];

%==================================================
% SUM OF THE ADMITTANCE MATRICES OF BRANCH N AND M
%==================================================
    ABCDYT = ABCDYN + ABCDYM;
%===================================
% ELEMENTS OF THE TOTAL ABCD MATRIX
%===================================
    ABCDT11=-ABCDYT(2,2)/ABCDYT(2,1);
    ABCDT12=-1/ABCDYT(2,1);
    ABCDT21=-(ABCDYT(1,1)*ABCDYT(2,2)- ABCDYT(1,2)*ABCDYT(2,1))/ABCDYT(2,1);
    ABCDT22=-(ABCDYT(1,1))/ABCDYT(2,1);
%============================
% BRANCH ADMITTANCE MATRIX M
%============================
    ABCDT=[ABCDT11 ABCDT12; ABCDT21 ABCDT22];
%====================================================================
% ELEMENT y11 AS A FUNCTION OF "s" OF THE MATRIX ADMITTANCE yT WHICH
% RELATES VIn TO VOut.
%====================================================================
    yMaux=[real(1/ABCDMabc(1,1))];
    yTaux=[real(1/ABCDT(1,1))];
    yM = cat(1,yM ,yMaux );
    yT = cat(1,yT , yTaux);
    length(yT)
end
%===================
% GRAPHIC SECTION
%===================
x = FRECUENCIAINICIAL:2:FRECUENCIAFINAL+FRECUENCIAINICIAL ;
datacase = [(x/1000)' 20*log(abs(yT))];
dlmwrite ("data_case_III.text", datacase)

figure(1)
plot(x/1000, 20*log(abs(yT)));
xlabel('Frequency (kHz)')
ylabel('Gain (db)')
grid on
